/* 5/01/98 This the sre_balancer procedure. 
   It replaces the BACKSRVR.SRF file in your goserve/lib directory.
   Assuming you've set LOADTHRESHOLD>0, it will be
   called upon every request, and will query the sre_balancer daemon
   (in SRE_BLNC.RXX).  

   On first request, it will initialize the sre-balancer daemon.
   After initialization (and on non-first requests), it will ask the 
   daemon to match the host_nickname, selector pair to the entries in 
   SRE_BLNC.INI.

*/
sref_backupserver:
parse arg bservers,loadthresh,hostnick,asel,servdir,verbose

if datatype(loadthresh)<>'NUM' then loadthresh=2

/* bservers is ignored, loadthresh is "minimum" -- if less then load_thresh,
   no balancing attempted */

if datatype(verbose)<>"NUM" then verbose=sref_value('VERBOSE',,'ini')
if servdir="" then servdir=sref_value('SERVDIR',,'ini')
transaction=extract('transaction')

if transaction=1 then do
   tfile=strip(servdir,'t','\')||'\SRE_BLNC.RXX'
   cfile=strip(servdir,'t','\')||'\SRE_BLNC.INI'
   params=cfile' 'verbose
   oo=sref_dmn_launch('SRE_LOAD_BALANCER',tfile,params)
   if word(oo,1)<0 then do
      aa=sref_error(' SRE_BLNC error: could not launch load balancer daemon: 'oo,1)
      return ""
   end /* do */
   if verbose>2 then call pmprintf_sref(' SRE_LOAD_BALANCER: daemon launched on thread 'oo)
end /* do */

nclients=extract('clients')

aflag=value('SREF_SRE_BALANCER',,'os2environment')
if aflag='' then return ''
if aflag=-1 then do
    'RESPONSE HTTP/1.1 500 Missing initialization file'
    'string Error: SRE_BLNC.INI not found '
    exit ''
end

/* check threshold */
if nclients<loadthresh then return ""

 aa=sref_dmn_exist('SRE_LOAD_BALANCER')
 if aa<1 then return ""                 /* does not exist, give up */

/* Ask daemon for match to hostnick// asel */
aparam=hostnick','asel
ares=sref_dmn_ask('SRE_LOAD_BALANCER',aparam,30)   /* wait 30 seconds */
if verbose>3 then call pmprintf(' Load balancer results='ares)
if ares="" then do              /* some kind of error, or no match */
   return ""                    /* don't load balance */
end /* do */

/* ares will be of form:
    minclients , url_list , priority_list
*/ 
parse var ares mincli ',' url_list ',' priority_list
mincli=strip(mincli)
if mincli>nclients then return ""         /* # clients < load balancing threshold */

if verbose>3 then call pmprintf_sref(' SRE_Load_balancer: '||words(url_list)||' possible destinations for 'asel)

if url_list="" then return ""  /* just in case ... */

if priority_list=' ' then do
   igot=random(1,words(url_list))
end /* do */
else do
   tmp=priority_list                       /*compute normalizer weight */
   sump=0
   do until tmp=""
      parse var tmp aa tmp
      sump=sump+aa
   end /* do */
   iyow=random(1,sump)

   sump=0 ; igot=1                         /* weighted random choooser */
   do ii=1 to words(priority_list)
      aa=strip(word(priority_list,ii))
      sump=sump+aa
      if sump>=iyow then do
        igot=ii ; leave
     end /* do */
  end /* do */
end


auri=strip(word(url_list,igot))   

if verbose>3 then call pmprintf_sref(' SRE_Load_balancer: redirect to 'auri)

auri=strip(auri,'t','/')
auri2='http://'||auri||'/'||asel
yow=value('SREF_PREFIX',,'os2environment')
if yow='' then do
  'RESPONSE HTTP/1.0 302 Moved Temporarily'  /* Set HTTP response line */
  'Header NoAuto Add Date: '||sref_gmt()
  'Header Add Server: 'server()||'; SRE-balancer '
end /* do */
else do
  'RESPONSE HTTP/1.0 302 Moved Temporarily'  /* Set HTTP response line */
end
'HEADER ADD Location:' auri2
'STRING  Redirect to backup server at '|| auri  /* perfunctory message to client */
return ' Redirected to '||auri                  /* back to srefiltr */


